package com.ruoyi.jjh.declaration.controller;

import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.jjh.declaration.entity.dto.BmsTemplateQueryDto;
import com.ruoyi.jjh.declaration.entity.dto.BmsTemplateRecordAddDto;
import com.ruoyi.jjh.declaration.entity.dto.BmsTemplateRecordQueryDto;
import com.ruoyi.jjh.declaration.entity.dto.BmsTemplateRecordUpdateDto;
import com.ruoyi.jjh.declaration.entity.vo.BmsTemplateRecordQueryVo;
import com.ruoyi.jjh.declaration.service.IBmsTemplateRecordService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 在线模板Controller
 * 
 * @author farben
 * @date 2023-09-07
 */
@RestController
@RequestMapping("/system/templateRecord")
public class BmsTemplateRecordController extends BaseController {
    @Autowired
    private IBmsTemplateRecordService bmsTemplateRecordService;

    /**
     * 查询在线模板列表
     */
//    @RequiresPermissions("system:templateRecord:list")
    @GetMapping("/list")
    public TableDataInfo list(BmsTemplateRecordQueryDto bmsTemplateRecordQueryDto) {
        startPage();
        List<BmsTemplateRecordQueryVo> list = bmsTemplateRecordService.selectBmsTemplateRecordList(bmsTemplateRecordQueryDto);
        return getDataTable(list);
    }

    @GetMapping("/templateNameList")
    public TableDataInfo templateNameList(BmsTemplateQueryDto bmsTemplateQueryDto){
        List<BmsTemplateRecordQueryVo> list = bmsTemplateRecordService.selectBmsTemplateNameList(bmsTemplateQueryDto);
        return getDataTable(list);
    }

    /**
     * 导出在线模板列表
     */
//    @RequiresPermissions("system:templateRecord:export")
    @Log(title = "在线模板", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BmsTemplateRecordQueryDto bmsTemplateRecordQueryDto) {
        List<BmsTemplateRecordQueryVo> list = bmsTemplateRecordService.selectBmsTemplateRecordList(bmsTemplateRecordQueryDto);
        ExcelUtil<BmsTemplateRecordQueryVo> util = new ExcelUtil<BmsTemplateRecordQueryVo>(BmsTemplateRecordQueryVo.class);
        util.exportExcel(response, list, "在线模板数据");
    }

    /**
     * 获取在线模板详细信息
     */
//    @RequiresPermissions("system:templateRecord:query")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return success(bmsTemplateRecordService.selectBmsTemplateRecordById(id));
    }

    /**
     * 新增在线模板
     */
//    @RequiresPermissions("system:templateRecord:add")
    @Log(title = "在线模板", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BmsTemplateRecordAddDto bmsTemplateRecordAddDto) {
        return toAjax(bmsTemplateRecordService.insertBmsTemplateRecord(bmsTemplateRecordAddDto));
    }

    /**
     * 修改在线模板
     */
//    @RequiresPermissions("system:templateRecord:edit")
    @Log(title = "在线模板", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BmsTemplateRecordUpdateDto bmsTemplateRecordUpdateDto) {
        return toAjax(bmsTemplateRecordService.updateBmsTemplateRecord(bmsTemplateRecordUpdateDto));
    }

    /**
     * 删除在线模板
     */
//    @RequiresPermissions("system:templateRecord:remove")
    @Log(title = "在线模板", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(bmsTemplateRecordService.deleteBmsTemplateRecordByIds(ids));
    }
}
