package com.ruoyi.jjh.declaration.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.jjh.declaration.entity.BmsBigStrongAward;
import com.ruoyi.jjh.declaration.entity.BmsDeclarationRecords;
import com.ruoyi.jjh.declaration.entity.BmsFundingDetail;
import com.ruoyi.jjh.declaration.entity.dto.BmsBigStrongAwardAddDto;
import com.ruoyi.jjh.declaration.entity.dto.BmsBigStrongAwardUpdateDto;
import com.ruoyi.jjh.declaration.entity.dto.BmsFundingDetailAddDto;
import com.ruoyi.jjh.declaration.entity.vo.BmsBigStrongAwardQueryVo;
import com.ruoyi.jjh.declaration.entity.vo.DeclarationRecordsVo;
import com.ruoyi.jjh.declaration.mapper.BmsBigStrongAwardMapper;
import com.ruoyi.jjh.declaration.service.IBmsBigStrongAwardService;
import com.ruoyi.jjh.declaration.service.IBmsDeclarationRecordsService;
import com.ruoyi.jjh.declaration.service.IBmsFundingDetailService;
import com.ruoyi.jjh.declaration.service.ICommonService;
import com.ruoyi.jjh.ent.entity.JProject;
import com.ruoyi.jjh.ent.service.JProjectService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.Year;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 做大做强奖详情Service业务层处理
 *
 * @author farben
 * @date 2023-08-25
 */
@Service
@Lazy
public class BmsBigStrongAwardServiceImpl extends ServiceImpl<BmsBigStrongAwardMapper, BmsBigStrongAward> implements IBmsBigStrongAwardService {

    @Autowired
    private ICommonService iCommonService;

    @Autowired
    private IBmsFundingDetailService iBmsFundingDetailService;

    @Autowired
    private IBmsDeclarationRecordsService iBmsDeclarationRecordsService;

    @Resource
    private JProjectService jProjectService;
    /**
     * 查询做大做强奖详情
     *
     * @param id 做大做强奖详情主键
     * @return 做大做强奖详情
     */
    @Override
    public BmsBigStrongAwardQueryVo selectBmsBigStrongAwardById(Long id) {
        return baseMapper.selectBmsBigStrongAwardById(id);
    }

    /**
     * 查询做大做强奖详情列表
     *
     * @param bmsBigStrongAward 做大做强奖详情
     * @return 做大做强奖详情
     */
    @Override
    public List<BmsBigStrongAward> selectBmsBigStrongAwardList(BmsBigStrongAward bmsBigStrongAward) {
        return baseMapper.selectBmsBigStrongAwardList(bmsBigStrongAward);
    }

    /**
     * 新增做大做强奖详情
     *
     * @param bmsBigStrongAwardAddDto 做大做强奖详情
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int insertBmsBigStrongAward(BmsBigStrongAwardAddDto bmsBigStrongAwardAddDto) {

        // check 企业信息
        Long enterpriseId = iCommonService.checkEnterprise(bmsBigStrongAwardAddDto.getCreditCode(), bmsBigStrongAwardAddDto.getTemplateRecordId());
        // check 申报信息
        iCommonService.checkDeclaration(bmsBigStrongAwardAddDto.getCreditCode()
                , bmsBigStrongAwardAddDto.getTemplateRecordId());
        if (enterpriseId == 0) {
            return 0;
        }
        BmsBigStrongAward bmsBigStrongAward = new BmsBigStrongAward();
        BeanUtil.copyProperties(bmsBigStrongAwardAddDto, bmsBigStrongAward);
        bmsBigStrongAward.setEnterpriseId(enterpriseId);
        // 新增做大做强奖详情
        int num = baseMapper.insert(bmsBigStrongAward);

        List<BmsFundingDetailAddDto> fundingDetailList = bmsBigStrongAwardAddDto.getFundingDetailList();
        List<BmsFundingDetail> fundingInfoList = new ArrayList<>();
        fundingDetailList.forEach(x -> {
            BmsFundingDetail fundingInfo = new BmsFundingDetail();
            BeanUtil.copyProperties(x, fundingInfo);
            fundingInfo.setType(2);
            fundingInfo.setTemplateRecordId(bmsBigStrongAwardAddDto.getTemplateRecordId());
            fundingInfo.setDetailId(bmsBigStrongAward.getId());
            fundingInfoList.add(fundingInfo);
        });
        // 批量新增资金信息
        iBmsFundingDetailService.saveOrUpdateBatch(fundingInfoList);
        // 新增申请记录
        BmsDeclarationRecords declarationRecords = new BmsDeclarationRecords();
        BeanUtil.copyProperties(bmsBigStrongAwardAddDto, declarationRecords);
        declarationRecords.setMatter("提交做大做强奖补申请");
        declarationRecords.setDetailId(bmsBigStrongAward.getId());
        declarationRecords.setEnterpriseId(enterpriseId);
        declarationRecords.setYear(String.valueOf(Year.now().getValue()));
        iBmsDeclarationRecordsService.save(declarationRecords);

        Map<String, String> allJson = getAllJson(declarationRecords.getId());

        //新增项目
        JProject jP = new JProject();
        jP.setProjectYear(declarationRecords.getYear());
        jP.setProjectName(declarationRecords.getProjectName());
        jP.setDeclareUnit(allJson.get("declareUnit"));
        jP.setProjectClassify(2);
        jP.setStatus(1);
        jP.setOtherJson(allJson.get("otherJson"));
        jP.setFileJson(allJson.get("fileJson"));
        jProjectService.saveProject(jP,declarationRecords.getId());

        // 申请审批
        if (bmsBigStrongAwardAddDto.getStatus() == 1) {
            // 审批状态
            iCommonService.insertAuditRecord(declarationRecords.getId(), 0);
        }

        return num;
    }

    @Override
    public Map<String,String> getAllJson(Long id) {
        Map<String,String> jsonMap = new HashMap<>();
        //获取该申报记录的详情
        DeclarationRecordsVo declarationRecordsVo = iBmsDeclarationRecordsService.selectBmsDeclarationRecordsById(id);
        //申报的企业信息JSON
        String js1= JSONObject.toJSONString(declarationRecordsVo.getBmsEnterpriseBasicInfo());
        //在线申报信息JSON
        String js2 = JSONObject.toJSONString(declarationRecordsVo.getBmsDeclarationRecords());
        //做大做强
        BmsBigStrongAwardQueryVo p1 = declarationRecordsVo.getBmsBigStrongAwardQueryVo();
        //做大做强对象JSON
        String js3 = JSONObject.toJSONString(p1);
        jsonMap.put("otherJson",jProjectService.allJsonString(js1, js2, js3, jProjectService.getJsonString(p1.getFundingDetailList())));
        jsonMap.put("fileJson","{" +
                '"' + "企业近3年发展情况及未来发展计划" + '"' + ":" + '"' + p1.getDevelopmentPlan() + '"' +
                "}");
        jsonMap.put("declareUnit",declarationRecordsVo.getBmsEnterpriseBasicInfo().getEnterpriseName());
        jsonMap.put("jjhProjectId", String.valueOf(declarationRecordsVo.getBmsDeclarationRecords().getJjhProjectId()));
        return jsonMap;
    }



    /**
     * 修改做大做强奖详情
     *
     * @param bmsBigStrongAwardUpdateDto 做大做强奖详情
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int updateBmsBigStrongAward(BmsBigStrongAwardUpdateDto bmsBigStrongAwardUpdateDto) {
        int num = 0;
        try {
            iBmsDeclarationRecordsService.updateDeclarationRecords(bmsBigStrongAwardUpdateDto.getDeclarationRecordsId()
                    , bmsBigStrongAwardUpdateDto.getProjectId(), bmsBigStrongAwardUpdateDto.getProjectName()
                    , bmsBigStrongAwardUpdateDto.getStatus());
            num = baseMapper.updateBmsBigStrongAward(bmsBigStrongAwardUpdateDto);
            List<BmsFundingDetail> fundingDetailList = bmsBigStrongAwardUpdateDto.getFundingDetailList();
            iBmsFundingDetailService.saveOrUpdateBatch(fundingDetailList);
            // 申请审批
            if (bmsBigStrongAwardUpdateDto.getStatus() == 1) {
                Map<String, String> allJson = getAllJson(bmsBigStrongAwardUpdateDto.getDeclarationRecordsId());
                //修改项目
                jProjectService.updateTheJson(allJson);
                // 审批状态
                iCommonService.insertAuditRecord(bmsBigStrongAwardUpdateDto.getDeclarationRecordsId(), 0);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return num;
    }

    /**
     * 批量删除做大做强奖详情
     *
     * @param ids 需要删除的做大做强奖详情主键
     * @return 结果
     */
    @Override
    public int deleteBmsBigStrongAwardByIds(Long[] ids) {
        return baseMapper.deleteBmsBigStrongAwardByIds(ids);
    }

    /**
     * 删除做大做强奖详情信息
     *
     * @param id 做大做强奖详情主键
     * @return 结果
     */
    @Override
    public int deleteBmsBigStrongAwardById(Long id) {
        return baseMapper.deleteBmsBigStrongAwardById(id);
    }
}
