package com.ruoyi.jjh.declaration.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.jjh.declaration.entity.BmsEnterpriseBasicInfo;
import com.ruoyi.jjh.declaration.entity.BmsEnterpriseDirectory;
import com.ruoyi.jjh.declaration.entity.BmsTemplateRecord;
import com.ruoyi.jjh.declaration.entity.dto.BmsTemplateQueryDto;
import com.ruoyi.jjh.declaration.entity.dto.BmsTemplateRecordAddDto;
import com.ruoyi.jjh.declaration.entity.dto.BmsTemplateRecordQueryDto;
import com.ruoyi.jjh.declaration.entity.dto.BmsTemplateRecordUpdateDto;
import com.ruoyi.jjh.declaration.entity.vo.BmsTemplateRecordQueryVo;
import com.ruoyi.jjh.declaration.entity.vo.BmsTemplateRecordVo;
import com.ruoyi.jjh.declaration.mapper.BmsTemplateRecordMapper;
import com.ruoyi.jjh.declaration.service.IBmsEnterpriseBasicInfoService;
import com.ruoyi.jjh.declaration.service.IBmsEnterpriseDirectoryService;
import com.ruoyi.jjh.declaration.service.IBmsTemplateRecordService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 在线模板Service业务层处理
 *
 * @author farben
 * @date 2023-09-07
 */
@Service
public class BmsTemplateRecordServiceImpl extends ServiceImpl<BmsTemplateRecordMapper, BmsTemplateRecord> implements IBmsTemplateRecordService {

    @Autowired
    private IBmsEnterpriseDirectoryService enterpriseDirectoryService;

    @Autowired
    private IBmsEnterpriseBasicInfoService iBmsEnterpriseBasicInfoService;

    /**
     * 查询在线模板
     *
     * @param id 在线模板主键
     * @return 在线模板
     */
    @Override
    public BmsTemplateRecordVo selectBmsTemplateRecordById(Long id) {
        BmsTemplateRecordVo bmsTemplateRecordVo = baseMapper.selectBmsTemplateRecordVoById(id);
        List<BmsEnterpriseDirectory> list = enterpriseDirectoryService.lambdaQuery()
                .eq(BmsEnterpriseDirectory::getTemplateRecordId, id).list();
        bmsTemplateRecordVo.setEnterpriseDirectoryList(list);
        return bmsTemplateRecordVo;
    }

    /**
     * 查询在线模板列表
     *
     * @param bmsTemplateRecordQueryDto 在线模板
     * @return 在线模板
     */
    @Override
    public List<BmsTemplateRecordQueryVo> selectBmsTemplateRecordList(BmsTemplateRecordQueryDto bmsTemplateRecordQueryDto) {
        return baseMapper.selectTemplateRecordVo(bmsTemplateRecordQueryDto);
    }

    @Override
    public List<BmsTemplateRecordQueryVo> selectTemplateRecordList(String creditCode) {
        return baseMapper.selectTemplateRecordList(creditCode);
    }

    @Override
    public List<BmsTemplateRecordQueryVo> selectBmsTemplateNameList(BmsTemplateQueryDto bmsTemplateQueryDto) {
        Long enterpriseId = bmsTemplateQueryDto.getEnterpriseId();
        BmsEnterpriseBasicInfo bmsEnterpriseBasicInfo = iBmsEnterpriseBasicInfoService.selectBmsEnterpriseBasicInfoById(enterpriseId);
        if (BeanUtil.isNotEmpty(bmsEnterpriseBasicInfo)) {
            bmsTemplateQueryDto.setCreditCode(bmsEnterpriseBasicInfo.getCreditCode());
            return baseMapper.selectBmsTemplateNameList(bmsTemplateQueryDto);
        }
        return null;
    }

    /**
     * 新增在线模板
     *
     * @param bmsTemplateRecordAddDto 在线模板
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int insertBmsTemplateRecord(BmsTemplateRecordAddDto bmsTemplateRecordAddDto) {
        int num = 0;
        try {
            bmsTemplateRecordAddDto.setCreateTime(DateUtils.getNowDate());
            num = baseMapper.insertBmsTemplateRecord(bmsTemplateRecordAddDto);
            List<BmsEnterpriseDirectory> enterpriseDirectoryList = bmsTemplateRecordAddDto
                    .getEnterpriseDirectoryList();
            if (null != enterpriseDirectoryList && enterpriseDirectoryList.size() > 0) {
                enterpriseDirectoryList.forEach(x -> {
                    x.setTemplateRecordId(bmsTemplateRecordAddDto.getId());
                });
                enterpriseDirectoryService.saveOrUpdateBatch(enterpriseDirectoryList);
            } else {
                List<BmsEnterpriseBasicInfo> list = iBmsEnterpriseBasicInfoService.list();
                Collection<BmsEnterpriseDirectory> copyList = new ArrayList<>();
                for (BmsEnterpriseBasicInfo items : list) {
                    BmsEnterpriseDirectory bms = new BmsEnterpriseDirectory();
                    bms.setTemplateRecordId(bmsTemplateRecordAddDto.getId());
                    bms.setEnterpriseName(items.getEnterpriseName());
                    bms.setCreditCode(items.getCreditCode());
                    copyList.add(bms);
                }
                enterpriseDirectoryService.saveBatch(copyList);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return num;
    }

    /**
     * 修改在线模板
     *
     * @param bmsTemplateRecordUpdateDto 在线模板
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int updateBmsTemplateRecord(BmsTemplateRecordUpdateDto bmsTemplateRecordUpdateDto) {
        bmsTemplateRecordUpdateDto.setUpdateTime(DateUtils.getNowDate());
        int num = baseMapper.updateBmsTemplateRecord(bmsTemplateRecordUpdateDto);
        // 删除企业相关列表
        QueryWrapper<BmsEnterpriseDirectory> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("template_record_id", bmsTemplateRecordUpdateDto.getId());
        enterpriseDirectoryService.remove(queryWrapper);
        // 新增企业相关列表
        List<BmsEnterpriseDirectory> enterpriseDirectoryList = bmsTemplateRecordUpdateDto
                .getEnterpriseDirectoryList();
        if (null != enterpriseDirectoryList && enterpriseDirectoryList.size() > 0) {
            enterpriseDirectoryList.forEach(x -> {
                x.setTemplateRecordId(bmsTemplateRecordUpdateDto.getId());
            });
            enterpriseDirectoryService.saveOrUpdateBatch(enterpriseDirectoryList);
        }
        return num;
    }

    /**
     * 批量删除在线模板
     *
     * @param ids 需要删除的在线模板主键
     * @return 结果
     */
    @Override
    public int deleteBmsTemplateRecordByIds(Long[] ids) {
        return baseMapper.deleteBmsTemplateRecordByIds(ids);
    }

    /**
     * 删除在线模板信息
     *
     * @param id 在线模板主键
     * @return 结果
     */
    @Override
    public int deleteBmsTemplateRecordById(Long id) {
        return baseMapper.deleteBmsTemplateRecordById(id);
    }
}
